/**
 * Created by eason on 15/5/26.
 */
// ChartHandler.start
/**
 * 图表 事件封装
 * @class ChartHandler
 * @param {jQuery} dom 图表所在的jQuery节点
 * @param chartGlyph  当前dom中展示的图表
 * @returns {ChartHandler}  返回自身
 * @constructor
 */
function ChartHandler(dom, chartGlyph, backgroundGlyph) {
    var root = dom[0];
    var self = this;
    this._chartGlyph = chartGlyph;
    this._chartGlyph.handler = this;
    this._backgroundGlyph = backgroundGlyph;
    this.getTooltipDiv = function(){
        return _tooltipDiv;
    };

    var _tooltipDiv = null;
    var _tooltipHideTick = null;
    var _tooltipDisplayTick = null;
    var _onMoveCtxTick = null;

    var _lastHover = null;  // 上次 悬浮over中的元素.

    var _hasfound = false; // 是否找到底层的Glyph 用于拖动 点击等.

    function _init() {
        root.addEventListener('click', _clickHandler);
        _tooltipDiv = document.createElement('div');
        _tooltipDiv.onselectstart = function(){
            return false;
        }
        _tooltipDiv.onclick = function(event){
            _doHyperlink(event);
        }
        root.appendChild(_tooltipDiv);
    }

    /**
     * 点击事件
     * @param {event} event dom事件对象
     */
    function _clickHandler(event) {

        var mousePos = _fixedPosition(event);

        //单独的背景的事件处理
        if(self._backgroundGlyph){
            self._backgroundGlyph.doOnClick(mousePos);
        }

        _findOnMoveGlyph(mousePos);

        self.needRefresh = false;
        self.needAnimateRefresh = true;
        self.needRefreshLabel = false;
        self.hyperlink = null;

        self.toNext = false;
        self.returnBack = false;

        if (!_lastHover) {
            return;
        }

        // 直接从当前的悬浮over元素中,  响应点击事件.
        if (_lastHover && _lastHover.onClick) {
            _lastHover.onClick(mousePos, self);
            //组合地图
            if(_lastHover.dataPoint4Map){
                self.hyperlink = _lastHover.dataPoint4Map.hyperlink;
                self.toNext = true;
            }
        }

        if(self.needRefresh){
            if(self.needRefreshLabel && self._chartGlyph.plotGlyph){
                self._chartGlyph.plotGlyph.refreshDataPointLabel();
            }
            if(self.needAnimateRefresh){
                _refresh();
            }else{
                refreshWithoutAnimate();
            }
        }

        if (self.toNext) {
            layerToDataPointNext(_lastHover);
        } else if (self.returnBack) {
            returnBackToLayer();
        }

        //点击的时候响应数据点提示和选中状态
        if(_lastHover.jsType && _lastHover.jsType == "DataPoint"){
            _resetOnMoveCtxTick();
            _tooltipHideDelay();
            self._chartGlyph.clearOnMoveCtx();
            var ctx4OnMove = self._chartGlyph.getOnMoveCtx();
            ctx4OnMove._tooltipDiv = _tooltipDiv;
            ctx4OnMove._hyperlink = self.hyperlink;
            var plotGlyph = self._chartGlyph.plotGlyph;
            _lastHover.doOnMove(ctx4OnMove, mousePos, plotGlyph, self._chartGlyph.getMapOnMoveCtx());
        }
    }


    function layerToDataPointNext(dataPoint) {
        dataPoint = FR.pick(dataPoint.dataPoint4Map ,dataPoint);
        self._chartGlyph.layerToDataPointNext(dataPoint, self);
        if (!FR.Browser.isIE()) {
            doRefreshIfNeed();
        }
    }

    function returnBackToLayer() {
        self._chartGlyph.returnBackToLayer();
        doRefreshIfNeed();
    }

    function _doHyperlink(event) {
        if (self.hyperlink) {
            FR.doHyperlink(event, (new Function("return " + self.hyperlink))(), true);
        }
    }

    function _tooltipHideDelay(){
        clearTimeout(_tooltipHideTick);
        clearTimeout(_tooltipDisplayTick);
        _tooltipHideTick = setTimeout(function(){
            _tooltipDiv.style.opacity = 0;
        }, 2000);

        _tooltipDisplayTick = setTimeout(function(){
            _tooltipDiv.style.display = 'none';
        }, 2500);
    }

    function _resetOnMoveCtxTick(){
        clearTimeout(_onMoveCtxTick);
        _onMoveCtxTick = setTimeout(function(){
            self._chartGlyph.clearOnMoveCtx();
        }, 2000);
    }

    /**
     * 查找当前onmove的Glyph
     * @param mousePos
     * @private
     */
    function _findOnMoveGlyph(mousePos){
        _hasfound = false;
        _lastHover = null;
        self._hasfound = false;
        self._lastHover = null;
        self._chartGlyph.findClickOnGlyph(mousePos, self);
        _lastHover = self._lastHover;
        _hasfound = self._hasfound;
    }


    function doRefreshIfNeed() {
        if (self.needRefresh) {
            _refresh();
        }
    }

    function refreshWithoutAnimate() {
        self._chartGlyph.clearAll();
        self._chartGlyph.plotGlyph.refreshAxisGlyph();
        self._chartGlyph.draw(FR.Chart.ChartPaintConstants.ACTION_MODEL_NULL);
    }

    function _refresh() {
        self._chartGlyph.clearAll();
        self._chartGlyph.plotGlyph.refreshAxisGlyph();
        self._chartGlyph.draw(FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE);
    }

    function _fixedPosition(event) {
        var ev = event;
        if (event.targetTouches && event.targetTouches.length > 0) {
            ev = event.targetTouches[0];
        } else {
            if (event && event.touches) {
                if (event.touches.length > 0) {
                    ev = event.touches[0];
                } else if (event.changedTouches && event.changedTouches.length > 0) {
                    ev = event.changedTouches[0];
                }
            }
        }

        var pos = {x: 0, y: 0};
        pos.x += ev.clientX;
        pos.y += ev.clientY;
        var offset = dom.offset();// 在放缩时保持了原样.
        pos.x -= offset.left;
        pos.y -= offset.top;

        return pos;
    }

    _init();

    return self;
}

$.extend(ChartHandler.prototype, {
    reStart: function (chartGlyph) {
        this._chartGlyph = chartGlyph;
        this._chartGlyph.handler = this;
    },

    doRefresh: function () {
        this._chartGlyph.clearAll();
        this._chartGlyph.plotGlyph.refreshAxisGlyph();
        this._chartGlyph.draw(FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE);
    }
});

// ChartHandler.end